"""App Settings"""

# Alliance Auth (External Libs)
from app_utils.app_settings import clean_setting

# Set Naming on Auth Hook
KILLSTATS_APP_NAME = clean_setting("KILLSTATS_APP_NAME", "Killstats")

# If True you need to set up the Logger
KILLSTATS_LOGGER_USE = clean_setting("KILLSTATS_LOGGER_USE", False)

# zKillboard - https://zkillboard.com/
ZKILLBOARD_BASE_URL = "https://zkillboard.com/"
ZKILLBOARD_API_URL = "https://zkillboard.com/api/"

ZKILLBOARD_BASE_URL_REGEX = r"^http[s]?:\/\/zkillboard\.com\/"
ZKILLBOARD_KILLMAIL_URL_REGEX = r"^http[s]?:\/\/zkillboard\.com\/kill\/\d+\/"

# Caching Key for Caching System
STORAGE_BASE_KEY = "killstats_storage_"

# Max lifetime of killmails in temporary storage in seconds
KILLSTATS_STORAGE_LIFETIME = clean_setting("KILLSTATS_STORAGE_LIFETIME", 3_600 * 24 * 3)

# Max Killmails per run should not be higher then 500
KILLSTATS_MAX_KILLMAILS_PER_RUN = clean_setting("KILLSTATS_MAX_KILLMAILS_PER_RUN", 200)

# Max lifetime of API Cache (10 min)
KILLSTATS_API_CACHE_LIFETIME = clean_setting("KILLSTATS_API_CACHE_LIFETIME", 10)

# Tasks hard timeout
KILLSTATS_TASKS_TIMEOUT = clean_setting("KILLSTATS_TASKS_TIMEOUT", 1_800)

# Unique ID used to identify this server when fetching killmails from zKillboard
# Please note that the Queue ID is globally unique for all users of the zKillboard API
# Only use characters Example: "Gneuten9000"
KILLSTATS_QUEUE_ID = clean_setting("KILLSTATS_QUEUE_ID", "")

# Max duration to wait for new killmails from redisq in seconds
KILLSTATS_REDISQ_TTW = clean_setting("KILLSTATS_REDISQ_TTW", 5)

# Timeout for Lock to ensure atomic access to ZKB RedisQ
KILLSTATS_REDISQ_LOCK_TIMEOUT = clean_setting("KILLSTATS_REDISQ_LOCK_TIMEOUT", 5)



"""Forms for the killstats app."""

from django import forms
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _


def get_mandatory_form_label_text(text: str) -> str:
    """Label text for mandatory form fields"""

    required_marker = "<span class='form-required-marker'>*</span>"

    return mark_safe(
        f"<span class='form-field-required'>{text} {required_marker}</span>"
    )


class SingleKillmail(forms.Form):
    """Form for Add Single Killmails."""

    killmail_id = forms.IntegerField(
        required=True,
        min_value=1,
        label=get_mandatory_form_label_text(_("Killmail ID")),
    )



from unittest.mock import patch

from django.core.exceptions import ObjectDoesNotExist
from django.test import RequestFactory, TestCase
from django.urls import reverse

from allianceauth.authentication.models import UserProfile
from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
from app_utils.testing import create_user_from_evecharacter

from killstats.api.account_manager import AccountManager
from killstats.api.killstats.api_helper import get_alliances, get_corporations
from killstats.tests.testdata.load_allianceauth import load_allianceauth

MODULE_PATH = "killstats.api.account_manager"


class TestApiHelpers(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_allianceauth()
        cls.factory = RequestFactory()

        cls.user, cls.char_owner = create_user_from_evecharacter(
            1001,
            permissions=[
                "killstats.basic_access",
            ],
        )
        cls.user2, _ = create_user_from_evecharacter(
            1002,
            permissions=[
                "killstats.basic_access",
            ],
        )
        cls.user3, _ = create_user_from_evecharacter(
            1003,
        )
        cls.user4, _ = create_user_from_evecharacter(
            1004,
        )
        cls.user5, _ = create_user_from_evecharacter(
            1005,
        )
        cls.corp = EveCorporationInfo.objects.get(corporation_id=2001)

    def test_get_main_and_alts_all_char_in_chars(self):
        # given
        request = self.factory.get("/")
        request.user = self.user

        expected_char_list = [
            1001,
            1002,
            1003,
            1004,
            1005,
        ]

        account = AccountManager()
        # when
        __, char_list = account.get_mains_alts()

        # then
        self.assertEqual(char_list, expected_char_list)

    def test_no_corp_or_ally(self):

        request = self.factory.get(reverse("killstats:index"))
        request.user = self.user

        corp_list = get_corporations(request)

        expected_data = []

        self.assertEqual(corp_list, expected_data)

        ally_list = get_alliances(request)

        expected_data = []

        self.assertEqual(ally_list, expected_data)



import json
from unittest.mock import MagicMock, patch

from django.test import TestCase, override_settings

from killstats.managers.killmail_core import KillmailManager
from killstats.models.killstatsaudit import AlliancesAudit, CorporationsAudit
from killstats.tasks import (
    store_killmail,
)
from killstats.tests.testdata.generate_killstats import (
    create_allianceaudit_from_character_id,
    create_corporationaudit_from_character_id,
)
from killstats.tests.testdata.load_allianceauth import load_allianceauth
from killstats.tests.testdata.load_eveuniverse import load_eveuniverse

MODULE_PATH = "killstats.tasks"


def load_test_data(filename):
    with open(filename) as file:
        return json.load(file)


# @patch(MODULE_PATH + ".killmail_update_corp.apply_async", spec=True)
# @patch(MODULE_PATH + ".killmail_update_ally.apply_async", spec=True)
@override_settings(
    CELERY_ALWAYS_EAGER=True,
    CELERY_EAGER_PROPAGATES_EXCEPTIONS=True,
    APP_UTILS_OBJECT_CACHE_DISABLED=True,
)
class TestUpdateTasks(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        cls.audit = create_corporationaudit_from_character_id(1001)
        cls.audit2 = create_allianceaudit_from_character_id(1001)



"""
Killstats Audit Model
"""

from django.db import models
from django.utils.translation import gettext_lazy as _

from allianceauth.eveonline.models import (
    EveAllianceInfo,
    EveCharacter,
    EveCorporationInfo,
)

from killstats.hooks import get_extension_logger
from killstats.managers.killboardaudit_manager import (
    AllianceManager,
    CorporationManager,
)
from killstats.managers.killmail_core import KillmailManager

logger = get_extension_logger(__name__)


class CorporationsAudit(models.Model):

    corporation = models.OneToOneField(
        EveCorporationInfo,
        on_delete=models.CASCADE,
    )

    last_update = models.DateTimeField(auto_now=True)

    owner = models.ForeignKey(EveCharacter, on_delete=models.CASCADE)

    objects = CorporationManager()

    def __str__(self):
        return f"{self.corporation.corporation_name}'s Killstats Data"

    def process_killmail(self, killmail: KillmailManager):
        """Process the killmail for this corporation"""
        if (
            killmail.victim.corporation_id == self.corporation.corporation_id
            or self.corporation.corporation_id
            in killmail.attackers_distinct_corporation_ids()
        ):
            return True
        return False

    class Meta:
        default_permissions = ()


class AlliancesAudit(models.Model):

    alliance = models.OneToOneField(
        EveAllianceInfo,
        on_delete=models.CASCADE,
    )

    last_update = models.DateTimeField(auto_now=True)

    owner = models.ForeignKey(EveCharacter, on_delete=models.CASCADE)

    objects = AllianceManager()

    def __str__(self):
        return f"{self.alliance.alliance_name}'s Killstats Data"

    def process_killmail(self, killmail: KillmailManager):
        """Process the killmail for this corporation"""
        if (
            killmail.victim.alliance_id == self.alliance.alliance_id
            or self.alliance.alliance_id in killmail.attackers_distinct_alliance_ids()
        ):
            return True
        return False

    class Meta:
        default_permissions = ()



from http import HTTPStatus
from unittest.mock import Mock, patch

from django.contrib.sessions.middleware import SessionMiddleware
from django.test import RequestFactory, TestCase, override_settings
from django.urls import reverse
from esi.models import Token

from allianceauth.eveonline.models import (
    EveAllianceInfo,
)
from app_utils.testing import create_user_from_evecharacter

from killstats.models.killstatsaudit import AlliancesAudit
from killstats.tests.testdata.load_allianceauth import load_allianceauth
from killstats.views import (
    add_alliance,
)

MODULE_PATH = "killstats.views"


@patch(MODULE_PATH + ".messages")
@patch(MODULE_PATH + ".provider.get_alliance")
@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
class KillstatsAllianceAuditTest(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_allianceauth()

        cls.factory = RequestFactory()
        cls.user, cls.character_ownership = create_user_from_evecharacter(
            1001,
            permissions=[
                "killstats.basic_access",
                "killstats.admin_access",
            ],
        )

    def _add_alliance(self, user, token):
        request = self.factory.get(reverse("killstats:add_alliance"))
        request.user = user
        request.token = token
        middleware = SessionMiddleware(Mock())
        middleware.process_request(request)
        orig_view = add_alliance.__wrapped__.__wrapped__.__wrapped__
        return orig_view(request, token)

    @patch(MODULE_PATH + ".EveAllianceInfo.objects.get_or_create")
    def test_add_ally(self, mock_alliance, mock_provider, mock_messages):
        # given
        user = self.user
        token = user.token_set.get(character_id=1001)

        alliance = EveAllianceInfo.objects.create(
            alliance_id=9999,
            alliance_name="Test Alliance",
            alliance_ticker="TEST",
            executor_corp_id=2001,
        )
        mock_alliance.return_value = (alliance, True)
        mock_provider.return_value = Mock()

        # when
        response = self._add_alliance(user, token)

        # then
        alliance_audit = AlliancesAudit.objects.get(alliance__alliance_id=9999)

        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertEqual(response.url, reverse("killstats:alliance", args=[9999]))
        self.assertEqual(mock_messages.info.call_count, 1)
        self.assertEqual(alliance_audit.alliance.alliance_id, 9999)



from django.test import RequestFactory, TestCase
from django.urls import reverse
from eveuniverse.models import EveEntity, EveType

from allianceauth.eveonline.models import EveAllianceInfo, EveCorporationInfo
from app_utils.testing import create_user_from_evecharacter

from killstats.models.killboard import Killmail
from killstats.tests.testdata.generate_killmail import create_killmail
from killstats.tests.testdata.load_allianceauth import load_allianceauth
from killstats.tests.testdata.load_eveuniverse import load_eveuniverse


class KillstatsManagerTest(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()
        cls.factory = RequestFactory()

        cls.killmail = create_killmail(
            killmail_id=1,
            killmail_date="2025-10-01T00:00:00Z",
            victim_corporation_id=2001,
            victim_alliance_id=3001,
            victim_region_id=1001,
            victim_solar_system_id=2001,
            victim_position_x=1.0,
            victim_position_y=1.0,
            victim_position_z=1.0,
            victim=EveEntity.objects.get(id=1001),
            victim_ship=EveType.objects.get(id=670),
            victim_total_value=1000,
            victim_fitted_value=500,
            victim_destroyed_value=500,
            victim_dropped_value=500,
        )

    def test_visible_to_superuser(self):
        # given
        self.user, self.character_ownership = create_user_from_evecharacter(
            1001,
        )
        self.user.is_superuser = True
        self.user.save()
        # when
        result = Killmail.objects.visible_to(self.user)
        expected_result = Killmail.objects.all()
        # then
        self.assertEqual(list(result), list(expected_result))

    def test_visible_to_admin_access(self):
        # given
        self.user, self.character_ownership = create_user_from_evecharacter(
            1001,
            permissions=[
                "killstats.admin_access",
            ],
        )
        # when
        expected_result = Killmail.objects.all()
        result = Killmail.objects.visible_to(self.user)
        # then
        self.assertEqual(list(result), list(expected_result))

    def test_visible_to_no_access(self):
        # given
        self.user, self.character_ownership = create_user_from_evecharacter(
            1001,
        )
        # when
        expected_result = []
        result = Killmail.objects.visible_to(self.user)
        # then
        self.assertEqual(list(result), list(expected_result))

    def test_filter_structure_exclude_false(self):
        queryset = Killmail.objects.filter_structure(exclude=False)
        expected_count = Killmail.objects.filter(
            victim_ship__eve_group__eve_category_id=65
        ).count()
        self.assertEqual(queryset.count(), expected_count)

    def test_filter_structure_exclude_true(self):
        queryset = Killmail.objects.filter_structure(exclude=True)
        expected_count = Killmail.objects.exclude(
            victim_ship__eve_group__eve_category_id=65
        ).count()
        self.assertEqual(queryset.count(), expected_count)



"""App Views"""

from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.db.models.functions import ExtractYear
from django.shortcuts import get_object_or_404, redirect, render
from django.utils.translation import gettext_lazy as _
from esi.decorators import token_required

from allianceauth.eveonline.models import (
    EveAllianceInfo,
    EveCharacter,
    EveCorporationInfo,
)
from allianceauth.eveonline.providers import ObjectNotFound, provider

# AA Killstats
from killstats import __title__
from killstats.forms import SingleKillmail
from killstats.managers.killboard_manager import KillmailManager
from killstats.models.killboard import Killmail
from killstats.models.killstatsaudit import AlliancesAudit, CorporationsAudit
from killstats.tasks import store_killmail

from .hooks import get_extension_logger

logger = get_extension_logger(__name__)


@login_required
@permission_required("killstats.basic_access")
def killboard_index(request):
    return redirect(
        "killstats:corporation", request.user.profile.main_character.corporation_id
    )


@login_required
@permission_required("killstats.basic_access")
def corporation_view(request, corporation_id=None):
    if corporation_id is None:
        corporation_id = request.user.profile.main_character.corporation_id

    years = (
        Killmail.objects.filter(victim_corporation_id=corporation_id)
        .annotate(year=ExtractYear("killmail_date"))
        .values_list("year", flat=True)
        .distinct()
        .order_by("-year")
    )[:5]

    context = {
        "title": "Corporation Killstats",
        "years": years,
        "entity_pk": corporation_id,
        "entity_type": "corporation",
    }
    return render(request, "killstats/killboard.html", context=context)


@login_required
@permission_required("killstats.basic_access")
def alliance_view(request, alliance_id=None):
    if alliance_id is None:
        try:
            alliance_id = request.user.profile.main_character.alliance_id
            if alliance_id is None:
                raise AttributeError
        except AttributeError:
            messages.error(request, "You do not have an alliance.")
            return redirect("killstats:index")

    years = (
        Killmail.objects.filter(victim_alliance_id=alliance_id)
        .annotate(year=ExtractYear("killmail_date"))
        .values_list("year", flat=True)
        .distinct()
        .order_by("-year")
    )[:5]

    context = {
        "title": "Alliance Killstats",
        "years": years,
        "entity_pk": alliance_id,
        "entity_type": "alliance",
    }
    return render(request, "killstats/killboard.html", context=context)


@login_required
@token_required(scopes=("publicData"))
@permission_required(["killstats.admin_access"])
def add_corp(request, token):
    char = get_object_or_404(EveCharacter, character_id=token.character_id)

    # Check if it is a NPC Corporation
    if char.corporation_id < 10_000_000:
        msg = "Cannot add NPC Corporation to Killstats"
        messages.error(request, msg)
        return redirect("killstats:index")

    corp, _ = EveCorporationInfo.objects.get_or_create(
        corporation_id=char.corporation_id,
        defaults={
            "member_count": 0,
            "corporation_ticker": char.corporation_ticker,
            "corporation_name": char.corporation_name,
        },
    )

    audit, __ = CorporationsAudit.objects.update_or_create(corporation=corp, owner=char)

    msg = (
        f"{audit.corporation.corporation_name} successfully added/updated to Killstats"
    )
    messages.info(request, msg)
    return redirect("killstats:corporation", corporation_id=corp.corporation_id)


@login_required
@token_required(scopes=("publicData"))
@permission_required(["killstats.admin_access"])
def add_alliance(request, token):
    char = get_object_or_404(EveCharacter, character_id=token.character_id)

    try:
        ally_data = provider.get_alliance(char.alliance_id)
        alliance, __ = EveAllianceInfo.objects.get_or_create(
            alliance_id=ally_data.id,
            defaults={
                "alliance_name": ally_data.name,
                "alliance_ticker": ally_data.ticker,
                "executor_corp_id": ally_data.executor_corp_id,
            },
        )
        audit, __ = AlliancesAudit.objects.update_or_create(
            alliance=alliance, owner=char
        )
        msg = _("{alliance_name} successfully added/updated to Killstats").format(
            alliance_name=audit.alliance.alliance_name,
        )
    except ObjectNotFound:
        msg = _("Failed to fetch Alliance data for {alliance_name}").format(
            alliance_name=char.alliance_name,
        )
        messages.warning(request, msg)
        return redirect("killstats:index")

    messages.info(request, msg)
    return redirect("killstats:alliance", alliance_id=alliance.alliance_id)


@login_required
@permission_required("killstats.basic_access")
def corporation_admin(request):
    """
    Corporation Admin
    """
    context = {
        "title": "Corporation Overview",
    }
    return render(request, "killstats/admin/corporation_admin.html", context=context)


@login_required
@permission_required("killstats.basic_access")
def alliance_admin(request):
    """
    Alliance Admin
    """
    context = {
        "title": "Alliance Overview",
    }
    return render(request, "killstats/admin/alliance_admin.html", context=context)


@login_required
@permission_required("killstats.basic_access")
def add_killmail(request):
    """
    Test
    """
    context = {
        "title": "Test",
        "forms": {
            "single_killmail": SingleKillmail(),
        },
    }
    form = SingleKillmail(request.POST)

    if form.is_valid():
        killmail_id = int(form.cleaned_data["killmail_id"])

        try:
            killmail = Killmail.objects.get(killmail_id=killmail_id)
            messages.error(
                request,
                _(
                    "Killmail {killmail_id} already exists. "
                    "Please check the killmail ID and try again."
                ).format(killmail_id=killmail_id),
            )
            return render(request, "killstats/killmail_add.html", context=context)
        except Killmail.DoesNotExist:
            killmail = KillmailManager.get_single_killmail(killmail_id)
            killmail.save()

            killmail = KillmailManager.get(killmail_id)

            if killmail:
                store_killmail.apply_async((killmail.id,))
                messages.success(
                    request,
                    _("Killmail {killmail_id} has been added to Killstats.").format(
                        killmail_id=killmail_id
                    ),
                )
    return render(request, "killstats/killmail_add.html", context=context)



from allianceauth.authentication.models import UserProfile
from allianceauth.eveonline.models import EveCharacter

from killstats.hooks import get_extension_logger

logger = get_extension_logger(__name__)


class AccountManager:
    def __init__(self):
        self.init_accounts()

    def init_accounts(self):
        self.accounts = UserProfile.objects.filter(
            main_character__isnull=False,
        ).select_related(
            "user__profile__main_character",
            "main_character__character_ownership",
            "main_character__character_ownership__user__profile",
            "main_character__character_ownership__user__profile__main_character",
        )

    def get_mains_alts(
        self,
    ) -> tuple[dict[int, dict[str, list[EveCharacter]]], list[int]]:
        """Get all members for given corporations/alliances"""
        characters = {}
        chars_list = set()

        for account in self.accounts:
            main = account.main_character
            alts_ids = account.user.character_ownerships.values_list(
                "character__character_id", flat=True
            )
            alts = EveCharacter.objects.filter(
                character_id__in=alts_ids,
            )

            if main and main.character_id not in characters:
                characters[main.character_id] = {"main": main, "alts": []}
                characters[main.character_id]["alts"].extend(alts)
                chars_list.add(main.character_id)
                chars_list.update(alts_ids)

        return characters, list(chars_list)



# Django
from django.db import models
from django.utils.translation import gettext_lazy as _
from eveuniverse.models import EveEntity, EveType

# AA Killstats
from killstats.hooks import get_extension_logger
from killstats.managers.killboard_manager import EveKillmailManager

logger = get_extension_logger(__name__)


class Killmail(models.Model):
    killmail_id = models.PositiveIntegerField(primary_key=True)
    killmail_date = models.DateTimeField(null=True, blank=True, max_length=0)
    victim = models.ForeignKey(EveEntity, on_delete=models.CASCADE, null=True)
    victim_ship = models.ForeignKey(EveType, on_delete=models.CASCADE, null=True)
    victim_corporation_id = models.PositiveIntegerField()
    victim_alliance_id = models.PositiveIntegerField(null=True, blank=True)
    hash = models.CharField(max_length=255, unique=True)
    # Value Infos
    victim_total_value = models.PositiveBigIntegerField(null=True, blank=True)
    victim_fitted_value = models.PositiveBigIntegerField(null=True, blank=True)
    victim_destroyed_value = models.PositiveBigIntegerField(null=True, blank=True)
    victim_dropped_value = models.PositiveBigIntegerField(null=True, blank=True)
    # Location Infos
    victim_region_id = models.PositiveIntegerField(null=True, blank=True)
    victim_solar_system_id = models.PositiveIntegerField(null=True, blank=True)
    victim_position_x = models.FloatField(null=True, blank=True)
    victim_position_y = models.FloatField(null=True, blank=True)
    victim_position_z = models.FloatField(null=True, blank=True)

    objects = EveKillmailManager()

    def __str__(self):
        return f"Killmail {self.killmail_id} - {self.killmail_date} - {self.victim} - {self.victim_ship}"

    def get_or_unknown_victim_name(self):
        """Return the victim name or Unknown."""
        return self.victim.name if self.victim else _("Unknown")

    def get_or_unknown_victim_ship_id(self):
        """Return the victim ship ID or Unknown."""
        return self.victim_ship.id if self.victim_ship else 0

    def get_or_unknown_victim_ship_name(self):
        """Return the victim ship name or Unknown."""
        return self.victim_ship.name if self.victim_ship else _("Unknown")

    def evaluate_zkb_link(self):
        zkb = f"https://zkillboard.com/character/{self.victim.id}/"
        if self.victim.category == "corporation":
            zkb = f"https://zkillboard.com/corporation/{self.victim_corporation_id}/"
        if self.victim.category == "alliance":
            zkb = f"https://zkillboard.com/alliance/{self.victim_alliance_id}/"
        return zkb

    class Meta:
        default_permissions = ()


class Attacker(models.Model):
    killmail = models.ForeignKey(
        Killmail, on_delete=models.CASCADE, related_name="attacker_killmail"
    )
    character = models.ForeignKey(
        EveEntity,
        on_delete=models.CASCADE,
        related_name="attacker_character",
        null=True,
        blank=True,
    )
    corporation = models.ForeignKey(
        EveEntity,
        on_delete=models.CASCADE,
        related_name="attacker_corp",
        null=True,
        blank=True,
    )
    alliance = models.ForeignKey(
        EveEntity,
        on_delete=models.CASCADE,
        related_name="attacker_alliance",
        null=True,
        blank=True,
    )
    ship = models.ForeignKey(
        EveType,
        on_delete=models.CASCADE,
        related_name="attacker_ship",
        null=True,
        blank=True,
    )
    damage_done = models.IntegerField(null=True, blank=True)
    final_blow = models.BooleanField(null=True, blank=True)
    weapon_type_id = models.PositiveIntegerField(null=True, blank=True)
    security_status = models.FloatField(null=True, blank=True)

    def evaluate_attacker(self) -> tuple:
        """Return the attacker ID and Name."""
        if self.character is not None:
            return self.character.id, self.character.name
        if self.corporation is not None:
            return self.corporation.id, self.corporation.name
        if self.alliance is not None:
            return self.alliance.id, self.alliance.name
        return 0, _("Unknown")

    class Meta:
        default_permissions = ()



# Generated by Django 4.2.11 on 2024-09-26 21:34

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("eveuniverse", "0010_alter_eveindustryactivityduration_eve_type_and_more"),
        ("killstats", "0002_rename_killstatsaudit_corporationsaudit_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="Attacker",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "killmail",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="attacker_killmail",
                        to="killstats.killmail",
                    ),
                ),
                ("damage_done", models.IntegerField(blank=True, null=True)),
                ("final_blow", models.BooleanField(blank=True, null=True)),
                (
                    "weapon_type_id",
                    models.PositiveBigIntegerField(blank=True, null=True),
                ),
                ("security_status", models.FloatField(blank=True, null=True)),
                (
                    "alliance",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="attacker_alliance",
                        to="eveuniverse.eveentity",
                    ),
                ),
                (
                    "character",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="attacker_character",
                        to="eveuniverse.eveentity",
                    ),
                ),
                (
                    "corporation",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="attacker_corp",
                        to="eveuniverse.eveentity",
                    ),
                ),
                (
                    "ship",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="attacker_ship",
                        to="eveuniverse.evetype",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.RemoveField(
            model_name="killmail",
            name="attackers",
        ),
        migrations.AlterField(
            model_name="killmail",
            name="victim",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="eveuniverse.eveentity",
            ),
        ),
        migrations.DeleteModel(
            name="EveEntity",
        ),
    ]



from unittest.mock import patch

from django.test import TestCase

from app_utils.esi import EsiDailyDowntime

from killstats.decorators import when_esi_is_available
from killstats.hooks import get_extension_logger


class TestDecorators(TestCase):
    @patch("killstats.decorators.fetch_esi_status")
    @patch("killstats.decorators.IS_TESTING", new=False)
    def test_when_esi_is_available(self, mock_fetch_esi_status):
        # given
        @when_esi_is_available
        def trigger_esi_deco():
            return "Esi is Available"

        # when
        result = trigger_esi_deco()
        # then
        mock_fetch_esi_status.assert_called_once()
        self.assertEqual(result, "Esi is Available")

    @patch("killstats.decorators.fetch_esi_status", side_effect=EsiDailyDowntime)
    @patch("killstats.decorators.IS_TESTING", new=False)
    def test_when_esi_is_available_downtime(self, mock_fetch_esi_status):
        # given
        @when_esi_is_available
        def trigger_esi_deco():
            return "Daily Downtime detected. Aborting."

        # when
        result = trigger_esi_deco()
        # then
        self.assertIsNone(result)

    @patch("killstats.decorators.fetch_esi_status")
    @patch("killstats.decorators.IS_TESTING", new=True)
    def test_when_esi_is_available_is_test(self, mock_fetch_esi_status):
        # given
        @when_esi_is_available
        def trigger_esi_deco():
            return "Teesting Mode."

        # when
        result = trigger_esi_deco()
        # then
        self.assertEqual(result, "Teesting Mode.")

    def test_log_timing(self):
        # given
        from killstats.decorators import log_timing

        logger = get_extension_logger(__name__)

        @log_timing(logger)
        def trigger_log_timing():
            return "Log Timing"

        # when
        result = trigger_log_timing()
        # then
        self.assertEqual(result, "Log Timing")



"""App Tasks"""

from celery import chain as Chain
from celery import shared_task

# Django
from django.db import IntegrityError

# AA Killstatsp
from killstats import app_settings
from killstats.decorators import when_esi_is_available
from killstats.hooks import get_extension_logger
from killstats.managers.killboard_manager import KillmailManager
from killstats.models.killboard import Killmail
from killstats.models.killstatsaudit import AlliancesAudit, CorporationsAudit

logger = get_extension_logger(__name__)


@shared_task(timeout=app_settings.KILLSTATS_TASKS_TIMEOUT)
@when_esi_is_available
def run_zkb_redis(runs: int = 0):
    killmail = KillmailManager.create_from_zkb_redisq()

    if killmail:
        killmail.save()

        corps_qs = CorporationsAudit.objects.all()
        allys_qs = AlliancesAudit.objects.all()

        for corporation in corps_qs:
            run_tracker_corporation.delay(
                corporation_id=corporation.corporation.corporation_id,
                killmail_id=killmail.id,
            )

        for alliance in allys_qs:
            run_tracker_alliance.delay(
                alliance_id=alliance.alliance.alliance_id, killmail_id=killmail.id
            )

    total_killmails = runs + (1 if killmail else 0)

    if killmail and total_killmails < app_settings.KILLSTATS_MAX_KILLMAILS_PER_RUN:
        run_zkb_redis.delay(runs=runs + 1)
    else:
        logger.info(
            "Killboard runs completed. %s killmails received from zKB",
            total_killmails,
        )


@shared_task
@when_esi_is_available
def run_tracker_corporation(corporation_id: int, killmail_id: int) -> None:
    """Run the tracker for the given killmail"""
    corporation = CorporationsAudit.objects.get(
        corporation__corporation_id=corporation_id
    )
    killmail = KillmailManager.get(killmail_id)
    killmail_new = corporation.process_killmail(killmail)
    if killmail_new:
        Chain(
            store_killmail.si(killmail.id),
        ).delay()
        logger.debug(
            "%s: Start storing killmail for %s",
            killmail.id,
            corporation.corporation.corporation_name,
        )


@shared_task
@when_esi_is_available
def run_tracker_alliance(alliance_id: int, killmail_id: int) -> None:
    """Run the tracker for the given killmail"""
    alliance = AlliancesAudit.objects.get(alliance__alliance_id=alliance_id)
    killmail = KillmailManager.get(killmail_id)
    killmail_new = alliance.process_killmail(killmail)
    if killmail_new:
        Chain(
            store_killmail.si(killmail.id),
        ).delay()
        logger.debug(
            "%s: Start storing killmail for %s",
            killmail.id,
            alliance.alliance.alliance_name,
        )


@shared_task(timeout=app_settings.KILLSTATS_TASKS_TIMEOUT)
def store_killmail(killmail_id: int) -> None:
    """stores killmail as EveKillmail object"""
    killmail = KillmailManager.get(killmail_id)
    try:
        Killmail.objects.create_from_killmail(killmail)
    except IntegrityError:
        logger.warning(
            "%s: Failed to store killmail, because it already exists", killmail.id
        )
    else:
        logger.debug("%s: Stored killmail", killmail.id)



from django.test import TestCase
from django.utils import timezone
from eveuniverse.models import EveEntity, EveType

from killstats.tests.testdata.generate_killmail import create_attacker, create_killmail
from killstats.tests.testdata.load_allianceauth import load_allianceauth
from killstats.tests.testdata.load_eveuniverse import load_eveuniverse

MODULE_PATH = "killstats.models.killstatsaudit"


class TestAttackertModel(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()
        cls.killmail = create_killmail(
            killmail_id=119303113,
            killmail_date=timezone.now(),
            victim=EveEntity.objects.get(id=1001),
            victim_ship=EveType.objects.get(id=670),
            victim_corporation_id=2001,
            victim_alliance_id=3001,
            hash="hash",
            victim_total_value=1000,
            victim_fitted_value=500,
            victim_destroyed_value=500,
            victim_dropped_value=500,
            victim_region_id=1001,
            victim_solar_system_id=2001,
            victim_position_x=1.0,
            victim_position_y=1.0,
            victim_position_z=1.0,
        )
        cls.attacker = create_attacker(
            killmail=cls.killmail,
            character=EveEntity.objects.get(id=1000),
            corporation=EveEntity.objects.get(id=1000125),
            alliance=EveEntity.objects.get(id=3001),
            ship=EveType.objects.get(id=670),
            damage_done=500,
            final_blow=True,
        )

    def test_evaluate_attacker_id(self):
        self.assertEqual(self.attacker.evaluate_attacker(), (1000, "CONCORD"))
        self.attacker.character = None
        self.assertEqual(self.attacker.evaluate_attacker(), (1000125, "CONCORD"))
        self.attacker.corporation = None
        self.assertEqual(self.attacker.evaluate_attacker(), (3001, "Voices of War"))
        self.attacker.alliance = None
        self.assertEqual(self.attacker.evaluate_attacker(), (0, "Unknown"))



"""App Configuration"""

# Django
from django.apps import AppConfig

# AA Killstats
from killstats import __version__


class KillstatsrConfig(AppConfig):
    """App Config"""

    default_auto_field = "django.db.models.AutoField"
    name = "killstats"
    label = "killstats"
    verbose_name = f"Killstats v{__version__}"



"""API"""

from ninja import NinjaAPI

from django.db.models import Count, Q
from django.shortcuts import render

from killstats.api.killstats import api_helper
from killstats.hooks import get_extension_logger

# AA Killstats
from killstats.models.killboard import Attacker, Killmail

logger = get_extension_logger(__name__)


class KillboardStatsApiEndpoints:
    tags = ["Stats"]

    # pylint: disable=too-many-locals
    def __init__(self, api: NinjaAPI):
        self.register_endpoints(api)

    # pylint: disable=too-many-statements
    def register_endpoints(self, api: NinjaAPI):
        @api.get(
            "stats/top/10/month/{month}/year/{year}/{entity_type}/{entity_id}/",
            response={200: list[dict], 403: str},
            tags=self.tags,
        )
        def get_top_10_api(request, month, year, entity_type: str, entity_id: int):
            cache_name = f"top_10_{month}_{year}_{entity_type}_{entity_id}"
            output, cache_key = api_helper.cache_sytem(request, cache_name, entity_id)
            if cache_key:
                top_10 = api_helper.get_top_10(
                    request=request,
                    month=month,
                    year=year,
                    entity_type=entity_type,
                    entity_id=entity_id,
                )

                output = {"top10": top_10}
                api_helper.set_cache_key(cache_key, output)

            return render(
                request,
                "killstats/partials/modal/view_top_10.html",
                output,
            )

        @api.get(
            "stats/all/month/{month}/year/{year}/{entity_type}/{entity_id}/",
            response={200: dict, 403: str},
            tags=self.tags,
        )
        def get_all_stats(request, month, year, entity_type: str, entity_id: int):
            cache_name = f"stats_{month}_{year}_{entity_type}_{entity_id}"
            output, cache_key = api_helper.cache_sytem(request, cache_name, entity_id)
            entities = api_helper.get_entities(request, entity_type, entity_id)

            if cache_key:
                # Get All Killmails
                killmail = Killmail.objects.filter(
                    Q(victim_id__in=entities)
                    | Q(victim_corporation_id__in=entities)
                    | Q(victim_alliance_id__in=entities),
                    killmail_date__year=year,
                )

                # Get All Attackers
                attackers = Attacker.objects.filter(
                    Q(corporation_id__in=entities)
                    | Q(alliance_id__in=entities)
                    | Q(character_id__in=entities),
                    killmail__killmail_date__year=year,
                )

                # Get for the month
                attackers_month = attackers.filter(killmail__killmail_date__month=month)
                killmail_month = killmail.filter(killmail_date__month=month)

                alltime_killer = (
                    attackers.values("character_id", "character__name")
                    .annotate(
                        alltime_killer=Count("killmail__killmail_id", distinct=True)
                    )
                    .order_by("-alltime_killer", "character__name")
                )

                alltime_victim = (
                    killmail.values("victim_id", "victim__name")
                    .annotate(alltime_victim=Count("killmail_id", distinct=True))
                    .order_by("-alltime_victim", "victim__name")
                )

                top_ship = (
                    attackers_month.values("ship__id", "ship__name")
                    .annotate(count=Count("killmail__killmail_id", distinct=True))
                    .order_by("-count")
                )

                top_killer = (
                    attackers_month.values("character_id", "character__name").annotate(
                        top_killer=Count("character_id")
                    )
                ).order_by("-top_killer", "character__name")

                top_victim = (
                    killmail_month.values("victim_id", "victim__name").annotate(
                        top_victim=Count("victim_id")
                    )
                ).order_by("-top_victim", "victim__name")

                top_victim_ship = (
                    killmail_month.exclude(victim_ship__eve_group_id__in=[29])
                    .values("victim_ship_id", "victim_ship__name")
                    .annotate(top_victim_ship=Count("victim_ship_id"))
                ).order_by("-top_victim_ship", "victim_ship__name")

                highest_kill = attackers_month.values(
                    "killmail_id",
                    "killmail__victim_total_value",
                    "killmail__victim_ship__id",
                    "killmail__victim_ship__name",
                ).order_by("-killmail__victim_total_value")

                highest_loss = killmail_month.values(
                    "killmail_id",
                    "victim_total_value",
                    "victim_ship__id",
                    "victim_ship__name",
                ).order_by("-victim_total_value")

                output = {
                    "stats": {
                        "alltime_killer": alltime_killer.first(),
                        "alltime_victim": alltime_victim.first(),
                        "top_ship": top_ship.first(),
                        "top_victim_ship": top_victim_ship.first(),
                        "top_victim": top_victim.first(),
                        "top_killer": top_killer.first(),
                        "highest_kill": highest_kill.first(),
                        "highest_loss": highest_loss.first(),
                    }
                }
                api_helper.set_cache_key(cache_key, output)

            return output



"""App URLs"""

# Django
from django.urls import path, re_path

from killstats import views

# AA Killstats
from killstats.api import api

app_name: str = "killstats"

urlpatterns = [
    # Killstats
    path("index", views.killboard_index, name="index"),
    path(
        "corporation/<int:corporation_id>/", views.corporation_view, name="corporation"
    ),
    path("alliance/<int:alliance_id>/", views.alliance_view, name="alliance"),
    path("corporation_admin/", views.corporation_admin, name="corporation_admin"),
    path("alliance_admin/", views.alliance_admin, name="alliance_admin"),
    # -- Killstats Audit
    path("add_corp/", views.add_corp, name="add_corp"),
    path("add_alliance/", views.add_alliance, name="add_alliance"),
    path("add_killmail/", views.add_killmail, name="add_killmail"),
    # -- API System
    re_path(r"^api/", api.urls),
]



# Generated by Django 4.2.11 on 2024-10-14 11:52

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("killstats", "0004_migrate_attacker"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="general",
            options={
                "default_permissions": (),
                "managed": False,
                "permissions": (
                    ("basic_access", "Can access this app, Killstats."),
                    ("admin_access", "Has access to all killstats."),
                ),
            },
        ),
    ]



from killstats.models.killboard import Attacker, Killmail


def create_killmail(**kwargs) -> Killmail:
    """Create a Killmail"""
    params = {}
    params.update(kwargs)

    killmail = Killmail(**params)
    killmail.save()
    return killmail


def create_attacker(**kwargs) -> Attacker:
    """Create an Attacker"""
    params = {}
    params.update(kwargs)

    attacker = Attacker(**params)
    attacker.save()
    return attacker



from django.db import models

from killstats.hooks import get_extension_logger

logger = get_extension_logger(__name__)


class CorporationsAuditQuerySet(models.QuerySet):
    def visible_to(self, user):
        # superusers get all visible
        if user.is_superuser:
            logger.debug(
                "Returning all corps for superuser %s.",
                user,
            )
            return self

        # Admins get all visible
        if user.has_perm("killstats.admin_access"):
            logger.debug("Returning all corps for Admin %s.", user)
            return self

        try:
            char = user.profile.main_character
            assert char
            query = models.Q(corporation__corporation_id=char.corporation_id)

            logger.debug("Returning Main Corporation for %s", user)

            return self.filter(query)
        except AssertionError:
            logger.debug("User %s has no main character. Nothing visible.", user)
            return self.none()


class CorporationsAuditManager(models.Manager):
    def get_queryset(self):
        return CorporationsAuditQuerySet(self.model, using=self._db)

    def visible_to(self, user):
        return self.get_queryset().visible_to(user)


CorporationManager = CorporationsAuditManager.from_queryset(CorporationsAuditQuerySet)


class AlliancesAuditQuerySet(models.QuerySet):
    def visible_to(self, user):
        # superusers get all visible
        if user.is_superuser:
            logger.debug(
                "Returning all allys for superuser %s.",
                user,
            )
            return self

        # Admins get all visible
        if user.has_perm("killstats.admin_access"):
            logger.debug("Returning all allys for Admin %s.", user)
            return self

        try:
            char = user.profile.main_character
            assert char
            query = models.Q(alliance__alliance_id=char.alliance_id)

            logger.debug("Returning Main Alliance for %s", user)

            return self.filter(query)
        except AssertionError:
            logger.debug("User %s has no main character. Nothing visible.", user)
            return self.none()


class AlliancesAuditManager(models.Manager):
    def get_queryset(self):
        return AlliancesAuditQuerySet(self.model, using=self._db)

    def visible_to(self, user):
        return self.get_queryset().visible_to(user)


AllianceManager = AlliancesAuditManager.from_queryset(AlliancesAuditQuerySet)



"""API"""

from ninja import NinjaAPI

from django.http import JsonResponse

# AA Killstats
from killstats.api import schema
from killstats.api.killstats import api_helper
from killstats.hooks import get_extension_logger

logger = get_extension_logger(__name__)


class KillboardApiEndpoints:
    tags = ["Killboard"]

    def __init__(self, api: NinjaAPI):
        self.register_endpoints(api)

    def register_endpoints(self, api: NinjaAPI):
        # Killmails
        @api.get(
            "killmail/month/{month}/year/{year}/{entity_type}/{entity_id}/{mode}/",
            response={200: dict, 403: str},
            tags=self.tags,
        )
        # pylint: disable=too-many-positional-arguments
        def get_corporation_killmails(
            request, month, year, entity_type: str, entity_id: int, mode
        ):
            output = api_helper.get_killmails_data(
                request, month, year, entity_type, entity_id, mode
            )
            return JsonResponse(output, safe=False)

        # Hall of Fame/Shame
        @api.get(
            "halls/month/{month}/year/{year}/{entity_type}/{entity_id}/",
            response={200: list[schema.KillboardHall], 403: str},
            tags=self.tags,
        )
        def get_corporation_halls(
            request, month, year, entity_type: str, entity_id: int
        ):
            cache_name = f"hall_{month}_{year}_{entity_type}_{entity_id}"
            output, cache_key = api_helper.cache_sytem(request, cache_name, entity_id)

            if cache_key:
                output = api_helper.get_killstats_halls(
                    request, month, year, entity_type, entity_id
                )
                # Cache the output
                api_helper.set_cache_key(cache_key, output)
            return JsonResponse(output, safe=False)



from django.contrib.auth.models import User
from eveuniverse.models import EveType

from allianceauth.authentication.backends import StateBackend
from allianceauth.authentication.models import CharacterOwnership
from allianceauth.eveonline.models import EveCharacter
from allianceauth.tests.auth_utils import AuthUtils
from app_utils.testing import add_character_to_user

from killstats.models.killstatsaudit import AlliancesAudit, CorporationsAudit


def create_corporation(eve_character: EveCharacter, **kwargs) -> CorporationsAudit:
    """Create a CorporationAudit from EveCharacter"""
    params = {
        "corporation": eve_character.corporation,
        "owner": eve_character,
    }
    params.update(kwargs)
    corporation = CorporationsAudit(**params)
    corporation.save()
    return corporation


def create_user_from_evecharacter_with_access(
    character_id: int, disconnect_signals: bool = True
) -> tuple[User, CharacterOwnership]:
    """Create user with access from an existing eve character and use it as main."""
    auth_character = EveCharacter.objects.get(character_id=character_id)
    username = StateBackend.iterate_username(auth_character.character_name)
    user = AuthUtils.create_user(username, disconnect_signals=disconnect_signals)
    user = AuthUtils.add_permission_to_user_by_name(
        "killstats.basic_access", user, disconnect_signals=disconnect_signals
    )
    character_ownership = add_character_to_user(
        user,
        auth_character,
        is_main=True,
        scopes=["publicData"],
        disconnect_signals=disconnect_signals,
    )
    return user, character_ownership


def create_corporationaudit_from_character_id(
    character_id: int, **kwargs
) -> CorporationsAudit:
    """Create a Corporation Audit from a existing EveCharacter"""

    _, character_ownership = create_user_from_evecharacter_with_access(
        character_id, disconnect_signals=True
    )
    return create_corporation(character_ownership.character, **kwargs)


def create_allianceaudit_from_character_id(
    character_id: int, **kwargs
) -> AlliancesAudit:
    """Create a Alliance Audit from a existing EveCharacter"""

    _, character_ownership = create_user_from_evecharacter_with_access(
        character_id, disconnect_signals=True
    )
    return create_alliance(character_ownership.character, **kwargs)


def add_auth_character_to_user(
    user: User, character_id: int, disconnect_signals: bool = True
) -> CharacterOwnership:
    auth_character = EveCharacter.objects.get(character_id=character_id)
    return add_character_to_user(
        user,
        auth_character,
        is_main=False,
        scopes=["publicData"],
        disconnect_signals=disconnect_signals,
    )


def add_corporationaudit_character_to_user(
    user: User, character_id: int, disconnect_signals: bool = True, **kwargs
) -> CorporationsAudit:
    """Add a Character Audit Character to a user"""
    character_ownership = add_auth_character_to_user(
        user,
        character_id,
        disconnect_signals=disconnect_signals,
    )
    return create_corporation(character_ownership.character, **kwargs)


def create_alliance(eve_character: EveCharacter, **kwargs) -> AlliancesAudit:
    """Create a AllianceAudit from EveCharacter"""
    params = {
        "alliance": eve_character.alliance,
        "owner": eve_character,
    }
    params.update(kwargs)
    alliance = AlliancesAudit(**params)
    alliance.save()
    return alliance



from unittest.mock import Mock, patch

from django.contrib.sessions.middleware import SessionMiddleware
from django.test import RequestFactory, TestCase
from django.urls import reverse
from esi.models import Token

from allianceauth.eveonline.models import (
    EveCorporationInfo,
)
from app_utils.testing import create_user_from_evecharacter

from killstats.models.killstatsaudit import CorporationsAudit
from killstats.tests.testdata.load_allianceauth import load_allianceauth
from killstats.views import (
    add_corp,
)

MODULE_PATH = "killstats.views"


@patch(MODULE_PATH + ".messages")
class KillstatsAuditTest(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_allianceauth()

        cls.factory = RequestFactory()
        cls.user, cls.character_ownership = create_user_from_evecharacter(
            1001,
            permissions=[
                "killstats.basic_access",
                "killstats.admin_access",
            ],
        )

    def _add_corporation(self, user, token):
        request = self.factory.get(reverse("killstats:add_corp"))
        request.user = user
        request.token = token
        middleware = SessionMiddleware(Mock())
        middleware.process_request(request)
        orig_view = add_corp.__wrapped__.__wrapped__.__wrapped__
        return orig_view(request, token)



from django.test import TestCase
from django.utils import timezone
from eveuniverse.models import EveEntity, EveType

from killstats.models.killboard import Killmail
from killstats.tests.testdata.generate_killmail import create_killmail
from killstats.tests.testdata.generate_killstats import (
    create_alliance,
    create_corporation,
)
from killstats.tests.testdata.load_allianceauth import load_allianceauth
from killstats.tests.testdata.load_eveuniverse import load_eveuniverse

MODULE_PATH = "killstats.models.killstatsaudit"


class TestKillstatsAuditModel(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        cls.killmail = create_killmail(
            killmail_id=1,
            killmail_date=timezone.datetime(2023, 1, 30, 0, 0, 0, tzinfo=timezone.utc),
            victim=EveEntity.objects.get(id=1001),
            victim_ship=EveType.objects.get(id=10001),
            victim_corporation_id=2001,
            victim_alliance_id=None,
            hash="hash1",
            victim_total_value=1_500_000,
            victim_fitted_value=1_500_000,
            victim_destroyed_value=1_500_000,
            victim_dropped_value=1_500_000,
            victim_region_id=1001,
            victim_solar_system_id=2001,
            victim_position_x=1.0,
            victim_position_y=1.0,
            victim_position_z=1.0,
        )
        cls.killmail2 = create_killmail(
            killmail_id=2,
            killmail_date=timezone.now(),
            victim=EveEntity.objects.get(id=1001),
            victim_ship=EveType.objects.get(id=30001),
            victim_corporation_id=2001,
            victim_alliance_id=3001,
            hash="hash2",
            victim_total_value=1000,
            victim_fitted_value=500,
            victim_destroyed_value=500,
            victim_dropped_value=500,
            victim_region_id=1001,
            victim_solar_system_id=2001,
            victim_position_x=1.0,
            victim_position_y=1.0,
            victim_position_z=1.0,
        )
        cls.killmail3 = create_killmail(
            killmail_id=3,
            killmail_date=timezone.now(),
            victim=EveEntity.objects.get(id=1001),
            victim_ship=EveType.objects.get(id=670),
            victim_corporation_id=2001,
            victim_alliance_id=3001,
            hash="hash3",
            victim_total_value=1000,
            victim_fitted_value=500,
            victim_destroyed_value=500,
            victim_dropped_value=500,
            victim_region_id=1001,
            victim_solar_system_id=2001,
            victim_position_x=1.0,
            victim_position_y=1.0,
            victim_position_z=1.0,
        )

    def test_str(self):
        self.assertEqual(
            str(self.killmail),
            "Killmail 1 - 2023-01-30 00:00:00+00:00 - Gneuten - Victim Ship I",
        )

    def test_get_victim_name(self):
        expected_url = "Gneuten"
        self.assertEqual(self.killmail.get_or_unknown_victim_name(), expected_url)

    def test_get_victim_ship_name(self):
        expected_url = "Victim Ship I"
        self.assertEqual(self.killmail.get_or_unknown_victim_ship_name(), expected_url)

    def test_evaluate_zkb_link(self):
        expected_url = "https://zkillboard.com/character/1001/"
        self.assertEqual(self.killmail.evaluate_zkb_link(), expected_url)

        self.killmail.victim.category = "corporation"
        expected_url = "https://zkillboard.com/corporation/2001/"
        self.assertEqual(self.killmail.evaluate_zkb_link(), expected_url)

        self.killmail2.victim.category = "alliance"
        expected_url = "https://zkillboard.com/alliance/3001/"
        self.assertEqual(self.killmail2.evaluate_zkb_link(), expected_url)



from http import HTTPStatus
from unittest.mock import Mock, patch

from django.test import RequestFactory, TestCase
from django.urls import reverse

from killstats import views
from killstats.tests.testdata.generate_killstats import (
    create_user_from_evecharacter_with_access,
)
from killstats.tests.testdata.load_allianceauth import load_allianceauth

MODULE_PATH = "killstats.views"


class TestViewKillstatsAccess(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()

        cls.factory = RequestFactory()

        cls.user, cls.character_ownership = create_user_from_evecharacter_with_access(
            1001
        )

    def test_view_killboard_index(self):
        """Test view Killstats index."""
        # given
        request = self.factory.get(reverse("killstats:index"))
        request.user = self.user
        # when
        response = views.killboard_index(request)
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)

    def test_corporation_view(self):
        """Test Corporation View."""
        # given
        request = self.factory.get(reverse("killstats:corporation", args=[2001]))
        request.user = self.user
        # when
        response = views.corporation_view(request, 2001)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)

    def test_alliance_view(self):
        """Test Alliance View."""
        # given
        request = self.factory.get(reverse("killstats:alliance", args=[3001]))
        request.user = self.user
        # when
        response = views.alliance_view(request, 3001)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)

    def test_corporation_admin_view(self):
        """Test Corporation Overview."""
        # given
        request = self.factory.get(reverse("killstats:corporation_admin"))
        request.user = self.user
        # when
        response = views.corporation_admin(request)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)

    def test_alliance_admin_view(self):
        """Test Alliance Overview."""
        # given
        request = self.factory.get(reverse("killstats:alliance_admin"))
        request.user = self.user
        # when
        response = views.alliance_admin(request)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)



from django.template import Context, Template
from django.test import TestCase, override_settings

from killstats import __version__
from killstats.helpers.static_files import calculate_integrity_hash


class TestVersionedStatic(TestCase):
    @override_settings(DEBUG=False)
    def test_versioned_static(self):
        context = Context(dict_={"version": __version__})
        template_to_render = Template(
            template_string=(
                "{% load killstats %}"
                "{% killstats_static 'css/killstats.css' %}"
                "{% killstats_static 'js/killstats.js' %}"
            )
        )

        rendered_template = template_to_render.render(context=context)

        expected_static_css_src = (
            f'/static/killstats/css/killstats.css?v={context["version"]}'
        )
        expected_static_css_src_integrity = calculate_integrity_hash(
            "css/killstats.css"
        )
        expected_static_js_src = (
            f'/static/killstats/js/killstats.js?v={context["version"]}'
        )
        expected_static_js_src_integrity = calculate_integrity_hash("js/killstats.js")

        self.assertIn(member=expected_static_css_src, container=rendered_template)
        self.assertIn(
            member=expected_static_css_src_integrity, container=rendered_template
        )
        self.assertIn(member=expected_static_js_src, container=rendered_template)
        self.assertIn(
            member=expected_static_js_src_integrity, container=rendered_template
        )

    @override_settings(DEBUG=True)
    def test_versioned_static_with_debug_enabled(self) -> None:
        context = Context({"version": __version__})
        template_to_render = Template(
            template_string=(
                "{% load killstats %}" "{% killstats_static 'css/killstats.css' %}"
            )
        )

        rendered_template = template_to_render.render(context=context)

        expected_static_css_src = (
            f'/static/killstats/css/killstats.css?v={context["version"]}'
        )

        self.assertIn(member=expected_static_css_src, container=rendered_template)
        self.assertNotIn(member="integrity=", container=rendered_template)

    @override_settings(DEBUG=False)
    def test_invalid_file_type(self) -> None:
        context = Context({"version": __version__})
        template_to_render = Template(
            template_string=(
                "{% load killstats %}" "{% killstats_static 'invalid/invalid.txt' %}"
            )
        )

        with self.assertRaises(ValueError):
            template_to_render.render(context=context)
